@@ -1,14 +1,9 @@
-// Copyright (C) 2008-2012 NICTA (www.nicta.com.au)
-// Copyright (C) 2008-2012 Conrad Sanderson
+// Copyright (C) 2008-2013 NICTA (www.nicta.com.au)
+// Copyright (C) 2008-2013 Conrad Sanderson
 // 
-// This file is part of the Armadillo C++ library.
-// It is provided without any warranty of fitness
-// for any purpose. You can redistribute this file
-// and/or modify it under the terms of the GNU
-// Lesser General Public License (LGPL) as published
-// by the Free Software Foundation, either version 3
-// of the License or (at your option) any later version.
-// (see http://www.opensource.org/licenses for more info)
+// This Source Code Form is subject to the terms of the Mozilla Public
+// License, v. 2.0. If a copy of the MPL was not distributed with this
+// file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 
 //! \addtogroup op_dot
@@ -32,7 +27,7 @@ class op_dot
   direct_dot_arma(const uword n_elem, const eT* const A, const eT* const B);
   
   template<typename eT>
-  arma_hot arma_pure inline static typename arma_float_only<eT>::result
+  arma_hot arma_pure inline static typename arma_real_only<eT>::result
   direct_dot(const uword n_elem, const eT* const A, const eT* const B);
   
   template<typename eT>
@@ -51,16 +46,13 @@ class op_dot
   arma_hot inline static typename T1::elem_type apply(const T1& X, const T2& Y);
   
   template<typename T1, typename T2>
-  arma_hot inline static typename T1::elem_type apply_unwrap(const T1& X, const T2& Y);
+  arma_hot inline static typename  arma_not_cx<typename T1::elem_type>::result apply_proxy(const Proxy<T1>& PA, const Proxy<T2>& PB);
   
   template<typename T1, typename T2>
-  arma_hot inline static typename arma_not_cx<typename T1::elem_type>::result  apply_proxy(const T1& X, const T2& Y);
+  arma_hot inline static typename arma_cx_only<typename T1::elem_type>::result apply_proxy(const Proxy<T1>& PA, const Proxy<T2>& PB);
   
-  template<typename T1, typename T2>
-  arma_hot inline static typename arma_cx_only<typename T1::elem_type>::result apply_proxy(const T1& X, const T2& Y);
-  
-  template<typename eT>
-  arma_hot inline static eT dot_and_copy_row(eT* out, const Mat<eT>& A, const uword row, const eT* B_mem, const uword N);
+  template<typename eT, typename TA>
+  arma_hot inline static eT dot_and_copy_row(eT* out, const TA& A, const uword row, const eT* B_mem, const uword N);
   };
 
 
