@@ -15,6 +15,7 @@ template<typename T1>
 struct unwrap_default
   {
   typedef typename T1::elem_type eT;
+  typedef Mat<eT>                stored_type;
   
   inline
   unwrap_default(const T1& A)
@@ -31,7 +32,7 @@ struct unwrap_default
 template<typename T1>
 struct unwrap_fixed
   {
-  typedef typename T1::elem_type eT;
+  typedef T1 stored_type;
   
   inline explicit
   unwrap_fixed(const T1& A)
@@ -40,7 +41,7 @@ struct unwrap_fixed
     arma_extra_debug_sigprint();
     }
   
-  const Mat<eT>& M;
+  const T1& M;
   };
 
 
@@ -70,6 +71,8 @@ struct unwrap : public unwrap_redirect<T1, is_Mat_fixed<T1>::value >::result
 template<typename eT>
 struct unwrap< Mat<eT> >
   {
+  typedef Mat<eT> stored_type;
+  
   inline
   unwrap(const Mat<eT>& A)
     : M(A)
@@ -85,6 +88,8 @@ struct unwrap< Mat<eT> >
 template<typename eT>
 struct unwrap< Row<eT> >
   {
+  typedef Row<eT> stored_type;
+  
   inline
   unwrap(const Row<eT>& A)
     : M(A)
@@ -100,6 +105,8 @@ struct unwrap< Row<eT> >
 template<typename eT>
 struct unwrap< Col<eT> >
   {
+  typedef Col<eT> stored_type;
+
   inline
   unwrap(const Col<eT>& A)
     : M(A)
@@ -115,6 +122,8 @@ struct unwrap< Col<eT> >
 template<typename out_eT, typename T1, typename T2, typename glue_type>
 struct unwrap< mtGlue<out_eT, T1, T2, glue_type> >
   {
+  typedef Mat<out_eT> stored_type;
+  
   inline
   unwrap(const mtGlue<out_eT, T1, T2, glue_type>& A)
     : M(A)
@@ -130,6 +139,8 @@ struct unwrap< mtGlue<out_eT, T1, T2, glue_type> >
 template<typename out_eT, typename T1, typename op_type>
 struct unwrap< mtOp<out_eT, T1, op_type> >
   {
+  typedef Mat<out_eT> stored_type;
+  
   inline
   unwrap(const mtOp<out_eT, T1, op_type>& A)
     : M(A)
@@ -354,6 +365,7 @@ template<typename T1>
 struct unwrap_check_default
   {
   typedef typename T1::elem_type eT;
+  typedef Mat<eT>                stored_type;
   
   inline
   unwrap_check_default(const T1& A, const Mat<eT>&)
@@ -378,19 +390,20 @@ template<typename T1>
 struct unwrap_check_fixed
   {
   typedef typename T1::elem_type eT;
+  typedef T1                     stored_type;
   
   inline
   unwrap_check_fixed(const T1& A, const Mat<eT>& B)
-    : M_local( (&A == &B) ? new Mat<eT>(A) : 0 )
-    , M      ( (&A == &B) ? *M_local       : A )
+    : M_local( (&A == &B) ? new T1(A) : 0 )
+    , M      ( (&A == &B) ? *M_local  : A )
     {
     arma_extra_debug_sigprint();
     }
   
   inline
   unwrap_check_fixed(const T1& A, const bool is_alias)
-    : M_local( is_alias ? new Mat<eT>(A) : 0 )
-    , M      ( is_alias ? *M_local       : A )
+    : M_local( is_alias ? new T1(A) : 0 )
+    , M      ( is_alias ? *M_local  : A )
     {
     arma_extra_debug_sigprint();
     }
@@ -405,8 +418,8 @@ struct unwrap_check_fixed
   
   
   // the order below is important
-  const Mat<eT>* M_local;
-  const Mat<eT>& M;
+  const T1* M_local;
+  const T1& M;
   };
 
 
@@ -440,6 +453,8 @@ struct unwrap_check : public unwrap_check_redirect<T1, is_Mat_fixed<T1>::value >
 template<typename eT>
 struct unwrap_check< Mat<eT> >
   {
+  typedef Mat<eT> stored_type;
+  
   inline
   unwrap_check(const Mat<eT>& A, const Mat<eT>& B)
     : M_local( (&A == &B) ? new Mat<eT>(A) : 0 )
@@ -474,6 +489,8 @@ struct unwrap_check< Mat<eT> >
 template<typename eT>
 struct unwrap_check< Row<eT> >
   {
+  typedef Row<eT> stored_type;
+  
   inline
   unwrap_check(const Row<eT>& A, const Mat<eT>& B)
     : M_local( (&A == &B) ? new Row<eT>(A) : 0 )
@@ -508,6 +525,8 @@ struct unwrap_check< Row<eT> >
 template<typename eT>
 struct unwrap_check< Col<eT> >
   {
+  typedef Col<eT> stored_type;
+  
   inline
   unwrap_check(const Col<eT>& A, const Mat<eT>& B)
     : M_local( (&A == &B) ? new Col<eT>(A) : 0 )
@@ -693,6 +712,7 @@ template<typename T1>
 struct partial_unwrap_default
   {
   typedef typename T1::elem_type eT;
+  typedef Mat<eT>                stored_type;
   
   inline
   partial_unwrap_default(const T1& A)
@@ -714,6 +734,7 @@ template<typename T1>
 struct partial_unwrap_fixed
   {
   typedef typename T1::elem_type eT;
+  typedef T1                     stored_type;
   
   inline explicit
   partial_unwrap_fixed(const T1& A)
@@ -727,7 +748,7 @@ struct partial_unwrap_fixed
   static const bool do_trans = false;
   static const bool do_times = false;
   
-  const Mat<eT>& M;
+  const T1& M;
   };
 
 
@@ -756,6 +777,8 @@ struct partial_unwrap : public partial_unwrap_redirect<T1, is_Mat_fixed<T1>::val
 template<typename eT>
 struct partial_unwrap< Mat<eT> >
   {
+  typedef Mat<eT> stored_type;
+  
   inline
   partial_unwrap(const Mat<eT>& A)
     : M(A)
@@ -776,6 +799,8 @@ struct partial_unwrap< Mat<eT> >
 template<typename eT>
 struct partial_unwrap< Row<eT> >
   {
+  typedef Row<eT> stored_type;
+  
   inline
   partial_unwrap(const Row<eT>& A)
     : M(A)
@@ -788,7 +813,7 @@ struct partial_unwrap< Row<eT> >
   static const bool do_trans = false;
   static const bool do_times = false;
   
-  const Mat<eT>& M;
+  const Row<eT>& M;
   };
 
 
@@ -796,6 +821,8 @@ struct partial_unwrap< Row<eT> >
 template<typename eT>
 struct partial_unwrap< Col<eT> >
   {
+  typedef Col<eT> stored_type;
+  
   inline
   partial_unwrap(const Col<eT>& A)
     : M(A)
@@ -808,7 +835,7 @@ struct partial_unwrap< Col<eT> >
   static const bool do_trans = false;
   static const bool do_times = false;
   
-  const Mat<eT>& M;
+  const Col<eT>& M;
   };
 
 
@@ -818,10 +845,11 @@ struct partial_unwrap< Col<eT> >
 template<typename eT>
 struct partial_unwrap< subview_col<eT> >
   {
+  typedef Col<eT> stored_type;
+  
   inline
   partial_unwrap(const subview_col<eT>& A)
-    : M  ( const_cast<eT*>( A.colptr(0) ), A.n_rows, 1, false, false )
-    //, ref( A )
+    : M  ( const_cast<eT*>( A.colptr(0) ), A.n_rows, false, false )
     {
     arma_extra_debug_sigprint();
     }
@@ -831,10 +859,7 @@ struct partial_unwrap< subview_col<eT> >
   static const bool do_trans = false;
   static const bool do_times = false;
   
-  const Mat<eT> M;
-  
-  //// prevents the compiler from potentially deleting the subview object before we're done with it
-  //const subview_col<eT>& ref;
+  const Col<eT> M;
   };
 
 
@@ -843,6 +868,7 @@ template<typename T1>
 struct partial_unwrap_htrans_default
   {
   typedef typename T1::elem_type eT;
+  typedef Mat<eT>                stored_type;
   
   inline
   partial_unwrap_htrans_default(const Op<T1, op_htrans>& A)
@@ -864,6 +890,7 @@ template<typename T1>
 struct partial_unwrap_htrans_fixed
   {
   typedef typename T1::elem_type eT;
+  typedef T1                     stored_type;
   
   inline explicit
   partial_unwrap_htrans_fixed(const Op<T1, op_htrans>& A)
@@ -877,7 +904,7 @@ struct partial_unwrap_htrans_fixed
   static const bool do_trans = true;
   static const bool do_times = false;
   
-  const Mat<eT>& M;
+  const T1& M;
   };
 
 
@@ -905,6 +932,8 @@ struct partial_unwrap< Op<T1, op_htrans> > : public partial_unwrap_htrans_redire
 template<typename eT>
 struct partial_unwrap< Op< Mat<eT>, op_htrans> >
   {
+  typedef Mat<eT> stored_type;
+  
   inline
   partial_unwrap(const Op< Mat<eT>, op_htrans>& A)
     : M(A.m)
@@ -925,6 +954,8 @@ struct partial_unwrap< Op< Mat<eT>, op_htrans> >
 template<typename eT>
 struct partial_unwrap< Op< Row<eT>, op_htrans> >
   {
+  typedef Row<eT> stored_type;
+  
   inline
   partial_unwrap(const Op< Row<eT>, op_htrans>& A)
     : M(A.m)
@@ -937,7 +968,7 @@ struct partial_unwrap< Op< Row<eT>, op_htrans> >
   static const bool do_trans = true;
   static const bool do_times = false;
   
-  const Mat<eT>& M;
+  const Row<eT>& M;
   };
 
 
@@ -945,6 +976,8 @@ struct partial_unwrap< Op< Row<eT>, op_htrans> >
 template<typename eT>
 struct partial_unwrap< Op< Col<eT>, op_htrans> >
   {
+  typedef Col<eT> stored_type;
+  
   inline
   partial_unwrap(const Op< Col<eT>, op_htrans>& A)
     : M(A.m)
@@ -957,7 +990,7 @@ struct partial_unwrap< Op< Col<eT>, op_htrans> >
   static const bool do_trans = true;
   static const bool do_times = false;
   
-  const Mat<eT>& M;
+  const Col<eT>& M;
   };
 
 
@@ -967,10 +1000,11 @@ struct partial_unwrap< Op< Col<eT>, op_htrans> >
 template<typename eT>
 struct partial_unwrap< Op< subview_col<eT>, op_htrans> >
   {
+  typedef Col<eT> stored_type;
+  
   inline
   partial_unwrap(const Op< subview_col<eT>, op_htrans>& A)
-    : M  ( const_cast<eT*>( A.m.colptr(0) ), A.m.n_rows, 1, false, false )
-    //, ref( A )
+    : M  ( const_cast<eT*>( A.m.colptr(0) ), A.m.n_rows, false, false )
     {
     arma_extra_debug_sigprint();
     }
@@ -980,10 +1014,7 @@ struct partial_unwrap< Op< subview_col<eT>, op_htrans> >
   static const bool do_trans = true;
   static const bool do_times = false;
   
-  const Mat<eT> M;
-  
-  //// prevents the compiler from potentially deleting the subview object before we're done with it
-  //const Op< subview_col<eT>, op_htrans>& ref;
+  const Col<eT> M;
   };
 
 
@@ -992,6 +1023,7 @@ template<typename T1>
 struct partial_unwrap_htrans2_default
   {
   typedef typename T1::elem_type eT;
+  typedef Mat<eT>                stored_type;
   
   inline
   partial_unwrap_htrans2_default(const Op<T1, op_htrans2>& A)
@@ -1015,6 +1047,7 @@ template<typename T1>
 struct partial_unwrap_htrans2_fixed
   {
   typedef typename T1::elem_type eT;
+  typedef T1                     stored_type;
   
   inline explicit
   partial_unwrap_htrans2_fixed(const Op<T1, op_htrans2>& A)
@@ -1029,8 +1062,8 @@ struct partial_unwrap_htrans2_fixed
   static const bool do_trans = true;
   static const bool do_times = true;
   
-  const eT      val;
-  const Mat<eT>& M;
+  const eT  val;
+  const T1& M;
   };
 
 
@@ -1058,6 +1091,8 @@ struct partial_unwrap< Op<T1, op_htrans2> > : public partial_unwrap_htrans2_redi
 template<typename eT>
 struct partial_unwrap< Op< Mat<eT>, op_htrans2> >
   {
+  typedef Mat<eT> stored_type;
+  
   inline
   partial_unwrap(const Op< Mat<eT>, op_htrans2>& A)
     : val(A.aux)
@@ -1080,6 +1115,8 @@ struct partial_unwrap< Op< Mat<eT>, op_htrans2> >
 template<typename eT>
 struct partial_unwrap< Op< Row<eT>, op_htrans2> >
   {
+  typedef Row<eT> stored_type;
+  
   inline
   partial_unwrap(const Op< Row<eT>, op_htrans2>& A)
     : val(A.aux)
@@ -1094,7 +1131,7 @@ struct partial_unwrap< Op< Row<eT>, op_htrans2> >
   static const bool do_times = true;
   
   const eT       val;
-  const Mat<eT>& M;
+  const Row<eT>& M;
   };
 
 
@@ -1102,6 +1139,8 @@ struct partial_unwrap< Op< Row<eT>, op_htrans2> >
 template<typename eT>
 struct partial_unwrap< Op< Col<eT>, op_htrans2> >
   {
+  typedef Col<eT> stored_type;
+  
   inline
   partial_unwrap(const Op< Col<eT>, op_htrans2>& A)
     : val(A.aux)
@@ -1116,7 +1155,7 @@ struct partial_unwrap< Op< Col<eT>, op_htrans2> >
   static const bool do_times = true;
   
   const eT       val;
-  const Mat<eT>& M;
+  const Col<eT>& M;
   };
 
 
@@ -1126,11 +1165,12 @@ struct partial_unwrap< Op< Col<eT>, op_htrans2> >
 template<typename eT>
 struct partial_unwrap< Op< subview_col<eT>, op_htrans2> >
   {
+  typedef Col<eT> stored_type;
+  
   inline
   partial_unwrap(const Op< subview_col<eT>, op_htrans2>& A)
     : val( A.aux )
-    , M  ( const_cast<eT*>( A.m.colptr(0) ), A.m.n_rows, 1, false, false )
-    //, ref( A )
+    , M  ( const_cast<eT*>( A.m.colptr(0) ), A.m.n_rows, false, false )
     {
     arma_extra_debug_sigprint();
     }
@@ -1141,10 +1181,7 @@ struct partial_unwrap< Op< subview_col<eT>, op_htrans2> >
   static const bool do_times = true;
   
   const eT      val;
-  const Mat<eT> M;
-
-  //// prevents the compiler from potentially deleting the subview object before we're done with it
-  //const Op< subview_col<eT>, op_htrans2>& ref;
+  const Col<eT> M;
   };
 
 
@@ -1153,6 +1190,7 @@ template<typename T1>
 struct partial_unwrap_scalar_times_default
   {
   typedef typename T1::elem_type eT;
+  typedef Mat<eT>                stored_type;
   
   inline
   partial_unwrap_scalar_times_default(const eOp<T1, eop_scalar_times>& A)
@@ -1177,6 +1215,7 @@ template<typename T1>
 struct partial_unwrap_scalar_times_fixed
   {
   typedef typename T1::elem_type eT;
+  typedef T1                     stored_type;
   
   inline explicit
   partial_unwrap_scalar_times_fixed(const eOp<T1, eop_scalar_times>& A)
@@ -1191,8 +1230,8 @@ struct partial_unwrap_scalar_times_fixed
   static const bool do_trans = false;
   static const bool do_times = true;
   
-  const eT       val;
-  const Mat<eT>& M;
+  const eT  val;
+  const T1& M;
   };
 
 
@@ -1224,6 +1263,8 @@ struct partial_unwrap< eOp<T1, eop_scalar_times> > : public partial_unwrap_scala
 template<typename eT>
 struct partial_unwrap< eOp<Mat<eT>, eop_scalar_times> >
   {
+  typedef Mat<eT> stored_type;
+  
   inline
   partial_unwrap(const eOp<Mat<eT>,eop_scalar_times>& A)
     : val(A.aux)
@@ -1246,6 +1287,8 @@ struct partial_unwrap< eOp<Mat<eT>, eop_scalar_times> >
 template<typename eT>
 struct partial_unwrap< eOp<Row<eT>, eop_scalar_times> >
   {
+  typedef Row<eT> stored_type;
+  
   inline
   partial_unwrap(const eOp<Row<eT>,eop_scalar_times>& A)
     : val(A.aux)
@@ -1260,7 +1303,7 @@ struct partial_unwrap< eOp<Row<eT>, eop_scalar_times> >
   static const bool do_times = true;
   
   const eT       val;
-  const Mat<eT>& M;
+  const Row<eT>& M;
   };
 
 
@@ -1268,6 +1311,8 @@ struct partial_unwrap< eOp<Row<eT>, eop_scalar_times> >
 template<typename eT>
 struct partial_unwrap< eOp<Col<eT>, eop_scalar_times> >
   {
+  typedef Col<eT> stored_type;
+  
   inline
   partial_unwrap(const eOp<Col<eT>,eop_scalar_times>& A)
     : val(A.aux)
@@ -1282,15 +1327,20 @@ struct partial_unwrap< eOp<Col<eT>, eop_scalar_times> >
   static const bool do_times = true;
   
   const eT       val;
-  const Mat<eT>& M;
+  const Col<eT>& M;
   };
 
 
 
+// TODO: struct partial_unwrap< eOp<subview_col<eT>, eop_scalar_times> >
+
+
+
 template<typename T1>
 struct partial_unwrap_neg_default
   {
   typedef typename T1::elem_type eT;
+  typedef Mat<eT>                stored_type;
   
   inline
   partial_unwrap_neg_default(const eOp<T1, eop_neg>& A)
@@ -1313,6 +1363,7 @@ template<typename T1>
 struct partial_unwrap_neg_fixed
   {
   typedef typename T1::elem_type eT;
+  typedef T1                     stored_type;
   
   inline explicit
   partial_unwrap_neg_fixed(const eOp<T1, eop_neg>& A)
@@ -1326,7 +1377,7 @@ struct partial_unwrap_neg_fixed
   static const bool do_trans = false;
   static const bool do_times = true;
   
-  const Mat<eT>& M;
+  const T1& M;
   };
 
 
@@ -1358,6 +1409,8 @@ struct partial_unwrap< eOp<T1, eop_neg> > : public partial_unwrap_neg_redirect<T
 template<typename eT>
 struct partial_unwrap< eOp<Mat<eT>, eop_neg> >
   {
+  typedef Mat<eT> stored_type;
+  
   inline
   partial_unwrap(const eOp<Mat<eT>,eop_neg>& A)
     : M(A.P.Q)
@@ -1378,6 +1431,8 @@ struct partial_unwrap< eOp<Mat<eT>, eop_neg> >
 template<typename eT>
 struct partial_unwrap< eOp<Row<eT>, eop_neg> >
   {
+  typedef Row<eT> stored_type;
+  
   inline
   partial_unwrap(const eOp<Row<eT>,eop_neg>& A)
     : M(A.P.Q)
@@ -1390,7 +1445,7 @@ struct partial_unwrap< eOp<Row<eT>, eop_neg> >
   static const bool do_trans = false;
   static const bool do_times = true;
   
-  const Mat<eT>& M;
+  const Row<eT>& M;
   };
 
 
@@ -1398,6 +1453,8 @@ struct partial_unwrap< eOp<Row<eT>, eop_neg> >
 template<typename eT>
 struct partial_unwrap< eOp<Col<eT>, eop_neg> >
   {
+  typedef Col<eT> stored_type;
+  
   inline
   partial_unwrap(const eOp<Col<eT>,eop_neg>& A)
     : M(A.P.Q)
@@ -1410,11 +1467,15 @@ struct partial_unwrap< eOp<Col<eT>, eop_neg> >
   static const bool do_trans = false;
   static const bool do_times = true;
   
-  const Mat<eT>& M;
+  const Col<eT>& M;
   };
 
 
 
+// TODO: struct partial_unwrap< eOp<subview_col<eT>, eop_neg> >
+
+
+
 //
 
 
@@ -1423,6 +1484,7 @@ template<typename T1>
 struct partial_unwrap_check_default
   {
   typedef typename T1::elem_type eT;
+  typedef Mat<eT>                stored_type;
   
   inline
   partial_unwrap_check_default(const T1& A, const Mat<eT>&)
@@ -1444,11 +1506,12 @@ template<typename T1>
 struct partial_unwrap_check_fixed
   {
   typedef typename T1::elem_type eT;
+  typedef T1                     stored_type;
   
   inline explicit
   partial_unwrap_check_fixed(const T1& A, const Mat<eT>& B)
-    : M_local( (&A == &B) ? new Mat<eT>(A) : 0 )
-    , M      ( (&A == &B) ? (*M_local)     : A )
+    : M_local( (&A == &B) ? new T1(A)  : 0 )
+    , M      ( (&A == &B) ? (*M_local) : A )
     {
     arma_extra_debug_sigprint();
     }
@@ -1466,8 +1529,8 @@ struct partial_unwrap_check_fixed
   static const bool do_trans = false;
   static const bool do_times = false;
   
-  const Mat<eT>* M_local;
-  const Mat<eT>& M;
+  const T1* M_local;
+  const T1& M;
   };
 
 
@@ -1497,6 +1560,8 @@ struct partial_unwrap_check : public partial_unwrap_check_redirect<T1, is_Mat_fi
 template<typename eT>
 struct partial_unwrap_check< Mat<eT> >
   {
+  typedef Mat<eT> stored_type;
+
   arma_hot inline
   partial_unwrap_check(const Mat<eT>& A, const Mat<eT>& B)
     : M_local ( (&A == &B) ? new Mat<eT>(A) : 0 )
@@ -1529,9 +1594,11 @@ struct partial_unwrap_check< Mat<eT> >
 template<typename eT>
 struct partial_unwrap_check< Row<eT> >
   {
+  typedef Row<eT> stored_type;
+  
   arma_hot inline
   partial_unwrap_check(const Row<eT>& A, const Mat<eT>& B)
-    : M_local ( (&A == &B) ? new Mat<eT>(A) : 0 )
+    : M_local ( (&A == &B) ? new Row<eT>(A) : 0 )
     , M       ( (&A == &B) ? (*M_local)     : A )
     {
     arma_extra_debug_sigprint();
@@ -1552,8 +1619,8 @@ struct partial_unwrap_check< Row<eT> >
   static const bool do_times = false;
   
   // the order below is important
-  const Mat<eT>* M_local;
-  const Mat<eT>& M;
+  const Row<eT>* M_local;
+  const Row<eT>& M;
   };
 
 
@@ -1561,9 +1628,11 @@ struct partial_unwrap_check< Row<eT> >
 template<typename eT>
 struct partial_unwrap_check< Col<eT> >
   {
+  typedef Col<eT> stored_type;
+  
   arma_hot inline
   partial_unwrap_check(const Col<eT>& A, const Mat<eT>& B)
-    : M_local ( (&A == &B) ? new Mat<eT>(A) : 0 )
+    : M_local ( (&A == &B) ? new Col<eT>(A) : 0 )
     , M       ( (&A == &B) ? (*M_local)     : A )
     {
     arma_extra_debug_sigprint();
@@ -1584,8 +1653,8 @@ struct partial_unwrap_check< Col<eT> >
   static const bool do_times = false;
   
   // the order below is important
-  const Mat<eT>* M_local;
-  const Mat<eT>& M;
+  const Col<eT>* M_local;
+  const Col<eT>& M;
   };
 
 
@@ -1595,10 +1664,11 @@ struct partial_unwrap_check< Col<eT> >
 template<typename eT>
 struct partial_unwrap_check< subview_col<eT> >
   {
+  typedef Col<eT> stored_type;
+  
   arma_hot inline
   partial_unwrap_check(const subview_col<eT>& A, const Mat<eT>& B)
-    : M  ( const_cast<eT*>( A.colptr(0) ), A.n_rows, 1, (&(A.m) == &B), false )
-    //, ref( A )
+    : M  ( const_cast<eT*>( A.colptr(0) ), A.n_rows, (&(A.m) == &B), false )
     {
     arma_extra_debug_sigprint();
     }
@@ -1608,10 +1678,7 @@ struct partial_unwrap_check< subview_col<eT> >
   static const bool do_trans = false;
   static const bool do_times = false;
   
-  const Mat<eT> M;
-
-  //// prevents the compiler from potentially deleting the subview object before we're done with it
-  //const subview_col<eT>& ref;
+  const Col<eT> M;
   };
 
 
@@ -1620,6 +1687,7 @@ template<typename T1>
 struct partial_unwrap_check_htrans_default
   {
   typedef typename T1::elem_type eT;
+  typedef Mat<eT>                stored_type;
   
   inline
   partial_unwrap_check_htrans_default(const Op<T1, op_htrans>& A, const Mat<eT>&)
@@ -1641,11 +1709,12 @@ template<typename T1>
 struct partial_unwrap_check_htrans_fixed
   {
   typedef typename T1::elem_type eT;
+  typedef T1                     stored_type;
   
   inline explicit
   partial_unwrap_check_htrans_fixed(const Op<T1, op_htrans>& A, const Mat<eT>& B)
-    : M_local( (&(A.m) == &B) ? new Mat<eT>(A.m) : 0   )
-    , M      ( (&(A.m) == &B) ? (*M_local)       : A.m )
+    : M_local( (&(A.m) == &B) ? new T1(A.m) : 0   )
+    , M      ( (&(A.m) == &B) ? (*M_local)  : A.m )
     {
     arma_extra_debug_sigprint();
     }
@@ -1663,8 +1732,8 @@ struct partial_unwrap_check_htrans_fixed
   static const bool do_trans = true;
   static const bool do_times = false;
   
-  const Mat<eT>* M_local;
-  const Mat<eT>& M;
+  const T1* M_local;
+  const T1& M;
   };
 
 
@@ -1695,6 +1764,8 @@ struct partial_unwrap_check< Op<T1, op_htrans> > : public partial_unwrap_check_h
 template<typename eT>
 struct partial_unwrap_check< Op< Mat<eT>, op_htrans> >
   {
+  typedef Mat<eT> stored_type;
+  
   arma_hot inline
   partial_unwrap_check(const Op< Mat<eT>, op_htrans>& A, const Mat<eT>& B)
     : M_local ( (&A.m == &B) ? new Mat<eT>(A.m) : 0   )
@@ -1726,9 +1797,11 @@ struct partial_unwrap_check< Op< Mat<eT>, op_htrans> >
 template<typename eT>
 struct partial_unwrap_check< Op< Row<eT>, op_htrans> >
   {
+  typedef Row<eT> stored_type;
+  
   arma_hot inline
   partial_unwrap_check(const Op< Row<eT>, op_htrans>& A, const Mat<eT>& B)
-    : M_local ( (&A.m == &B) ? new Mat<eT>(A.m) : 0   )
+    : M_local ( (&A.m == &B) ? new Row<eT>(A.m) : 0   )
     , M       ( (&A.m == &B) ? (*M_local)       : A.m )
     {
     arma_extra_debug_sigprint();
@@ -1748,8 +1821,8 @@ struct partial_unwrap_check< Op< Row<eT>, op_htrans> >
   static const bool do_times = false;
   
   // the order below is important
-  const Mat<eT>* M_local;
-  const Mat<eT>& M;
+  const Row<eT>* M_local;
+  const Row<eT>& M;
   };
 
 
@@ -1757,9 +1830,11 @@ struct partial_unwrap_check< Op< Row<eT>, op_htrans> >
 template<typename eT>
 struct partial_unwrap_check< Op< Col<eT>, op_htrans> >
   {
+  typedef Col<eT> stored_type;
+  
   arma_hot inline
   partial_unwrap_check(const Op< Col<eT>, op_htrans>& A, const Mat<eT>& B)
-    : M_local ( (&A.m == &B) ? new Mat<eT>(A.m) : 0   )
+    : M_local ( (&A.m == &B) ? new Col<eT>(A.m) : 0   )
     , M       ( (&A.m == &B) ? (*M_local)       : A.m )
     {
     arma_extra_debug_sigprint();
@@ -1779,8 +1854,8 @@ struct partial_unwrap_check< Op< Col<eT>, op_htrans> >
   static const bool do_times = false;
   
   // the order below is important
-  const Mat<eT>* M_local;
-  const Mat<eT>& M;
+  const Col<eT>* M_local;
+  const Col<eT>& M;
   };
 
 
@@ -1790,10 +1865,11 @@ struct partial_unwrap_check< Op< Col<eT>, op_htrans> >
 template<typename eT>
 struct partial_unwrap_check< Op< subview_col<eT>, op_htrans> >
   {
+  typedef Col<eT> stored_type;
+  
   arma_hot inline
   partial_unwrap_check(const Op< subview_col<eT>, op_htrans>& A, const Mat<eT>& B)
-    : M  ( const_cast<eT*>( A.m.colptr(0) ), A.m.n_rows, 1, (&(A.m.m) == &B), false )
-    //, ref( A )
+    : M  ( const_cast<eT*>( A.m.colptr(0) ), A.m.n_rows, (&(A.m.m) == &B), false )
     {
     arma_extra_debug_sigprint();
     }
@@ -1803,10 +1879,7 @@ struct partial_unwrap_check< Op< subview_col<eT>, op_htrans> >
   static const bool do_trans = true;
   static const bool do_times = false;
   
-  const Mat<eT> M;
-  
-  //// prevents the compiler from potentially deleting the subview object before we're done with it
-  //const Op< subview_col<eT>, op_htrans>& ref;
+  const Col<eT> M;
   };
 
 
@@ -1815,6 +1888,7 @@ template<typename T1>
 struct partial_unwrap_check_htrans2_default
   {
   typedef typename T1::elem_type eT;
+  typedef Mat<eT>                stored_type;
   
   inline
   partial_unwrap_check_htrans2_default(const Op<T1, op_htrans2>& A, const Mat<eT>&)
@@ -1839,12 +1913,13 @@ template<typename T1>
 struct partial_unwrap_check_htrans2_fixed
   {
   typedef typename T1::elem_type eT;
+  typedef T1                     stored_type;
   
   inline explicit
   partial_unwrap_check_htrans2_fixed(const Op<T1, op_htrans2>& A, const Mat<eT>& B)
     : val    (A.aux)
-    , M_local( (&(A.m) == &B) ? new Mat<eT>(A.m) : 0   )
-    , M      ( (&(A.m) == &B) ? (*M_local)       : A.m )
+    , M_local( (&(A.m) == &B) ? new T1(A.m) : 0   )
+    , M      ( (&(A.m) == &B) ? (*M_local)  : A.m )
     {
     arma_extra_debug_sigprint();
     }
@@ -1862,9 +1937,9 @@ struct partial_unwrap_check_htrans2_fixed
   static const bool do_trans = true;
   static const bool do_times = true;
   
-  const eT       val;
-  const Mat<eT>* M_local;
-  const Mat<eT>& M;
+  const eT  val;
+  const T1* M_local;
+  const T1& M;
   };
 
 
@@ -1895,6 +1970,8 @@ struct partial_unwrap_check< Op<T1, op_htrans2> > : public partial_unwrap_check_
 template<typename eT>
 struct partial_unwrap_check< Op< Mat<eT>, op_htrans2> >
   {
+  typedef Mat<eT> stored_type;
+  
   arma_hot inline
   partial_unwrap_check(const Op< Mat<eT>, op_htrans2>& A, const Mat<eT>& B)
     : val     (A.aux)
@@ -1928,10 +2005,12 @@ struct partial_unwrap_check< Op< Mat<eT>, op_htrans2> >
 template<typename eT>
 struct partial_unwrap_check< Op< Row<eT>, op_htrans2> >
   {
+  typedef Row<eT> stored_type;
+  
   arma_hot inline
   partial_unwrap_check(const Op< Row<eT>, op_htrans2>& A, const Mat<eT>& B)
     : val     (A.aux)
-    , M_local ( (&A.m == &B) ? new Mat<eT>(A.m) : 0   )
+    , M_local ( (&A.m == &B) ? new Row<eT>(A.m) : 0   )
     , M       ( (&A.m == &B) ? (*M_local)       : A.m )
     {
     arma_extra_debug_sigprint();
@@ -1952,8 +2031,8 @@ struct partial_unwrap_check< Op< Row<eT>, op_htrans2> >
   
   // the order below is important
   const eT       val;
-  const Mat<eT>* M_local;
-  const Mat<eT>& M;
+  const Row<eT>* M_local;
+  const Row<eT>& M;
   };
 
 
@@ -1961,10 +2040,12 @@ struct partial_unwrap_check< Op< Row<eT>, op_htrans2> >
 template<typename eT>
 struct partial_unwrap_check< Op< Col<eT>, op_htrans2> >
   {
+  typedef Col<eT> stored_type;
+  
   arma_hot inline
   partial_unwrap_check(const Op< Col<eT>, op_htrans2>& A, const Mat<eT>& B)
     : val     (A.aux)
-    , M_local ( (&A.m == &B) ? new Mat<eT>(A.m) : 0   )
+    , M_local ( (&A.m == &B) ? new Col<eT>(A.m) : 0   )
     , M       ( (&A.m == &B) ? (*M_local)       : A.m )
     {
     arma_extra_debug_sigprint();
@@ -1985,8 +2066,8 @@ struct partial_unwrap_check< Op< Col<eT>, op_htrans2> >
   
   // the order below is important
   const eT       val;
-  const Mat<eT>* M_local;
-  const Mat<eT>& M;
+  const Col<eT>* M_local;
+  const Col<eT>& M;
   };
 
 
@@ -1996,11 +2077,12 @@ struct partial_unwrap_check< Op< Col<eT>, op_htrans2> >
 template<typename eT>
 struct partial_unwrap_check< Op< subview_col<eT>, op_htrans2> >
   {
+  typedef Col<eT> stored_type;
+  
   arma_hot inline
   partial_unwrap_check(const Op< subview_col<eT>, op_htrans2>& A, const Mat<eT>& B)
     : val( A.aux )
-    , M  ( const_cast<eT*>( A.m.colptr(0) ), A.m.n_rows, 1, (&(A.m.m) == &B), false )
-    //, ref( A )
+    , M  ( const_cast<eT*>( A.m.colptr(0) ), A.m.n_rows, (&(A.m.m) == &B), false )
     {
     arma_extra_debug_sigprint();
     }
@@ -2011,10 +2093,7 @@ struct partial_unwrap_check< Op< subview_col<eT>, op_htrans2> >
   static const bool do_times = true;
   
   const eT      val;
-  const Mat<eT> M;
-
-  //// prevents the compiler from potentially deleting the subview object before we're done with it
-  //const Op< subview_col<eT>, op_htrans2>& ref;
+  const Col<eT> M;
   };
 
 
@@ -2023,6 +2102,7 @@ template<typename T1>
 struct partial_unwrap_check_scalar_times_default
   {
   typedef typename T1::elem_type eT;
+  typedef Mat<eT>                stored_type;
   
   inline
   partial_unwrap_check_scalar_times_default(const eOp<T1, eop_scalar_times>& A, const Mat<eT>&)
@@ -2047,12 +2127,13 @@ template<typename T1>
 struct partial_unwrap_check_scalar_times_fixed
   {
   typedef typename T1::elem_type eT;
+  typedef T1                     stored_type;
   
   inline explicit
   partial_unwrap_check_scalar_times_fixed(const eOp<T1, eop_scalar_times>& A, const Mat<eT>& B)
     : val    ( A.aux )
-    , M_local( (&(A.P.Q) == &B) ? new Mat<eT>(A.P.Q) : 0     )
-    , M      ( (&(A.P.Q) == &B) ? (*M_local)         : A.P.Q )
+    , M_local( (&(A.P.Q) == &B) ? new T1(A.P.Q) : 0     )
+    , M      ( (&(A.P.Q) == &B) ? (*M_local)    : A.P.Q )
     {
     arma_extra_debug_sigprint();
     }
@@ -2070,9 +2151,9 @@ struct partial_unwrap_check_scalar_times_fixed
   static const bool do_trans = false;
   static const bool do_times = true;
   
-  const eT       val;
-  const Mat<eT>* M_local;
-  const Mat<eT>& M;
+  const eT  val;
+  const T1* M_local;
+  const T1& M;
   };
 
 
@@ -2103,6 +2184,8 @@ struct partial_unwrap_check< eOp<T1, eop_scalar_times> > : public partial_unwrap
 template<typename eT>
 struct partial_unwrap_check< eOp<Mat<eT>, eop_scalar_times> >
   {
+  typedef Mat<eT> stored_type;
+  
   arma_hot inline
   partial_unwrap_check(const eOp<Mat<eT>,eop_scalar_times>& A, const Mat<eT>& B)
     : val    (A.aux)
@@ -2135,10 +2218,12 @@ struct partial_unwrap_check< eOp<Mat<eT>, eop_scalar_times> >
 template<typename eT>
 struct partial_unwrap_check< eOp<Row<eT>, eop_scalar_times> >
   {
+  typedef Row<eT> stored_type;
+  
   arma_hot inline
   partial_unwrap_check(const eOp<Row<eT>,eop_scalar_times>& A, const Mat<eT>& B)
     : val(A.aux)
-    , M_local( (&(A.P.Q) == &B) ? new Mat<eT>(A.P.Q) : 0     )
+    , M_local( (&(A.P.Q) == &B) ? new Row<eT>(A.P.Q) : 0     )
     , M      ( (&(A.P.Q) == &B) ? *M_local           : A.P.Q )
     {
     arma_extra_debug_sigprint();
@@ -2158,8 +2243,8 @@ struct partial_unwrap_check< eOp<Row<eT>, eop_scalar_times> >
   static const bool do_times = true;
   
   const eT       val;
-  const Mat<eT>* M_local;
-  const Mat<eT>& M;
+  const Row<eT>* M_local;
+  const Row<eT>& M;
   };
 
 
@@ -2167,10 +2252,12 @@ struct partial_unwrap_check< eOp<Row<eT>, eop_scalar_times> >
 template<typename eT>
 struct partial_unwrap_check< eOp<Col<eT>, eop_scalar_times> >
   {
+  typedef Col<eT> stored_type;
+  
   arma_hot inline
   partial_unwrap_check(const eOp<Col<eT>,eop_scalar_times>& A, const Mat<eT>& B)
     : val    ( A.aux )
-    , M_local( (&(A.P.Q) == &B) ? new Mat<eT>(A.P.Q) : 0     )
+    , M_local( (&(A.P.Q) == &B) ? new Col<eT>(A.P.Q) : 0     )
     , M      ( (&(A.P.Q) == &B) ? *M_local           : A.P.Q )
     {
     arma_extra_debug_sigprint();
@@ -2190,8 +2277,8 @@ struct partial_unwrap_check< eOp<Col<eT>, eop_scalar_times> >
   static const bool do_times = true;
   
   const eT       val;
-  const Mat<eT>* M_local;
-  const Mat<eT>& M;
+  const Col<eT>* M_local;
+  const Col<eT>& M;
   };
 
 
@@ -2201,11 +2288,12 @@ struct partial_unwrap_check< eOp<Col<eT>, eop_scalar_times> >
 template<typename eT>
 struct partial_unwrap_check< eOp<subview_col<eT>, eop_scalar_times> >
   {
+  typedef Col<eT> stored_type;
+  
   arma_hot inline
   partial_unwrap_check(const eOp<subview_col<eT>,eop_scalar_times>& A, const Mat<eT>& B)
     : val( A.aux )
-    , M  ( const_cast<eT*>( A.P.Q.colptr(0) ), A.P.Q.n_rows, 1, (&(A.P.Q.m) == &B), false )
-    //, ref( A )
+    , M  ( const_cast<eT*>( A.P.Q.colptr(0) ), A.P.Q.n_rows, (&(A.P.Q.m) == &B), false )
     {
     arma_extra_debug_sigprint();
     }
@@ -2216,10 +2304,7 @@ struct partial_unwrap_check< eOp<subview_col<eT>, eop_scalar_times> >
   static const bool do_times = true;
   
   const eT      val;
-  const Mat<eT> M;
-
-  //// prevents the compiler from potentially deleting the subview object before we're done with it
-  //const eOp<subview_col<eT>,eop_scalar_times>& ref;
+  const Col<eT> M;
   };
 
 
@@ -2228,6 +2313,7 @@ template<typename T1>
 struct partial_unwrap_check_neg_default
   {
   typedef typename T1::elem_type eT;
+  typedef Mat<eT>                stored_type;
   
   inline
   partial_unwrap_check_neg_default(const eOp<T1, eop_neg>& A, const Mat<eT>&)
@@ -2250,11 +2336,12 @@ template<typename T1>
 struct partial_unwrap_check_neg_fixed
   {
   typedef typename T1::elem_type eT;
+  typedef T1                     stored_type;
   
   inline explicit
   partial_unwrap_check_neg_fixed(const eOp<T1, eop_neg>& A, const Mat<eT>& B)
-    : M_local( (&(A.P.Q) == &B) ? new Mat<eT>(A.P.Q) : 0     )
-    , M      ( (&(A.P.Q) == &B) ? (*M_local)         : A.P.Q )
+    : M_local( (&(A.P.Q) == &B) ? new T1(A.P.Q) : 0     )
+    , M      ( (&(A.P.Q) == &B) ? (*M_local)    : A.P.Q )
     {
     arma_extra_debug_sigprint();
     }
@@ -2272,8 +2359,8 @@ struct partial_unwrap_check_neg_fixed
   static const bool do_trans = false;
   static const bool do_times = true;
   
-  const Mat<eT>* M_local;
-  const Mat<eT>& M;
+  const T1* M_local;
+  const T1& M;
   };
 
 
@@ -2304,6 +2391,8 @@ struct partial_unwrap_check< eOp<T1, eop_neg> > : public partial_unwrap_check_ne
 template<typename eT>
 struct partial_unwrap_check< eOp<Mat<eT>, eop_neg> >
   {
+  typedef Mat<eT> stored_type;
+  
   arma_hot inline
   partial_unwrap_check(const eOp<Mat<eT>,eop_neg>& A, const Mat<eT>& B)
     : M_local( (&(A.P.Q) == &B) ? new Mat<eT>(A.P.Q) : 0     )
@@ -2334,9 +2423,11 @@ struct partial_unwrap_check< eOp<Mat<eT>, eop_neg> >
 template<typename eT>
 struct partial_unwrap_check< eOp<Row<eT>, eop_neg> >
   {
+  typedef Row<eT> stored_type;
+  
   arma_hot inline
   partial_unwrap_check(const eOp<Row<eT>,eop_neg>& A, const Mat<eT>& B)
-    : M_local( (&(A.P.Q) == &B) ? new Mat<eT>(A.P.Q) : 0     )
+    : M_local( (&(A.P.Q) == &B) ? new Row<eT>(A.P.Q) : 0     )
     , M      ( (&(A.P.Q) == &B) ? *M_local           : A.P.Q )
     {
     arma_extra_debug_sigprint();
@@ -2355,8 +2446,8 @@ struct partial_unwrap_check< eOp<Row<eT>, eop_neg> >
   static const bool do_trans = false;
   static const bool do_times = true;
   
-  const Mat<eT>* M_local;
-  const Mat<eT>& M;
+  const Row<eT>* M_local;
+  const Row<eT>& M;
   };
 
 
@@ -2364,9 +2455,11 @@ struct partial_unwrap_check< eOp<Row<eT>, eop_neg> >
 template<typename eT>
 struct partial_unwrap_check< eOp<Col<eT>, eop_neg> >
   {
+  typedef Col<eT> stored_type;
+  
   arma_hot inline
   partial_unwrap_check(const eOp<Col<eT>,eop_neg>& A, const Mat<eT>& B)
-    : M_local( (&(A.P.Q) == &B) ? new Mat<eT>(A.P.Q) : 0     )
+    : M_local( (&(A.P.Q) == &B) ? new Col<eT>(A.P.Q) : 0     )
     , M      ( (&(A.P.Q) == &B) ? *M_local           : A.P.Q )
     {
     arma_extra_debug_sigprint();
@@ -2385,8 +2478,8 @@ struct partial_unwrap_check< eOp<Col<eT>, eop_neg> >
   static const bool do_trans = false;
   static const bool do_times = true;
   
-  const Mat<eT>* M_local;
-  const Mat<eT>& M;
+  const Col<eT>* M_local;
+  const Col<eT>& M;
   };
 
 
@@ -2396,9 +2489,11 @@ struct partial_unwrap_check< eOp<Col<eT>, eop_neg> >
 template<typename eT>
 struct partial_unwrap_check< eOp<subview_col<eT>, eop_neg> >
   {
+  typedef Col<eT> stored_type;
+  
   arma_hot inline
   partial_unwrap_check(const eOp<subview_col<eT>,eop_neg>& A, const Mat<eT>& B)
-    : M  ( const_cast<eT*>( A.P.Q.colptr(0) ), A.P.Q.n_rows, 1, (&(A.P.Q.m) == &B), false )
+    : M  ( const_cast<eT*>( A.P.Q.colptr(0) ), A.P.Q.n_rows, (&(A.P.Q.m) == &B), false )
     {
     arma_extra_debug_sigprint();
     }
@@ -2408,7 +2503,7 @@ struct partial_unwrap_check< eOp<subview_col<eT>, eop_neg> >
   static const bool do_trans = false;
   static const bool do_times = true;
   
-  const Mat<eT> M;
+  const Col<eT> M;
   };
 
 
